unit rDBGridSorter_ClientDS;

{<
  rDBGridSorter_ClientDS unit contains TrDBGridSorter_ClientDS - sorter for TCustomClientDataSet
  used by rDBGrid to sort Client datasets.

  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Oct 2006 - Jun 2013)
  @lastmod(Dec 2015)
}

interface

uses
  Windows, Messages, SysUtils, Graphics, Classes, Controls, Menus, Dialogs,
  StdCtrls, Forms, DB, Variants, rDBGrid,
  DBClient;

type
  { Sorter for TCustomClientDataSet - used by rDBGrid to sort any Client datasets }
  TrDBGridSorter_ClientDS = class(TrDBGridCustomSorter)
  protected
    function SetSort_CreateIndex(DataSet: TDataSet; const SortString: string; const SortFields: string; const DescendingFields: string): boolean; override;
  public
    { returns filter wildcard char for specified dataset type}
    function GetFilterWildCardChar: char; override;
    { returns filter like operator for specified dataset type}
    function GetFilterLikeOperator: string; override;
  end;

procedure Register; {< @exclude }

implementation

{$R *.res}
uses RosiCompGlobalCfg;

type
  THackCustomClientDataSet = class(TCustomClientDataSet)
  end;

procedure Register;
begin
  RegisterComponents('Rosi DB', [TrDBGridSorter_ClientDS]);
end;

function TrDBGridSorter_ClientDS.SetSort_CreateIndex(DataSet: TDataSet; const SortString: string; const SortFields: string; const DescendingFields: string)
  : boolean;
var
  DS: TCustomClientDataSet;
  idxName: string;
begin
  if (DataSet is TCustomClientDataSet) then
  begin
    DS:=DataSet as TCustomClientDataSet;
    if Trim(SortFields)='' then
      THackCustomClientDataSet(DS).IndexName:=''
    else
    begin
      // indexname can be max 31 chars long - use hash from fields names
      idxName:=GetIndexNameHash(SortFields + '_' + DescendingFields);
      if THackCustomClientDataSet(DS).IndexDefs.IndexOf(idxName) < 0 then
        DS.AddIndex(idxName, SortFields, [], DescendingFields);
      THackCustomClientDataSet(DS).IndexDefs.Update;
      THackCustomClientDataSet(DS).IndexName:=idxName;
    end;
    Result:=true;
  end
  else
    Result:= inherited;
end;

function TrDBGridSorter_ClientDS.GetFilterWildCardChar: char;
begin
  Result:='%';
end;

function TrDBGridSorter_ClientDS.GetFilterLikeOperator: string;
begin
  Result:='LIKE';
end;

end.
