unit f_main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, rStringGridEd, PropSaveMain, PropSaveGrids,
  rImprovedComps, rHTMLLabel, Masks;

type
  TMain = class(TForm)
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    GridsPropSave1: TGridsPropSave;
    mFList: TrStringsHolder;
    mFile: TrStringsHolder;
    gbClient: TrGroupBox;
    Label1: TLabel;
    eDir: TEdit;
    btSelDir: TButton;
    btRun: TButton;
    gr: TrStringGridEd;
    m: TMemo;
    chBackup: TCheckBox;
    btLoad: TrURLLabel;
    btSave: TrURLLabel;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    btClear: TrURLLabel;
    btTest: TButton;
    chLogFiles: TCheckBox;
    rHTMLLabel1: TrHTMLLabel;
    procedure btSelDirClick(Sender: TObject);
    procedure btRunClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btLoadClick(Sender: TObject);
    procedure btSaveClick(Sender: TObject);
  private
    BkDir: string;
    UpdMsg: string;
    UpdFiles, UpdProps: integer;
    procedure ProcDFM(fn: string; test: boolean);
    function ProcObj(ClassName, PropertyName, old, new: string; del: boolean; CompName, NewClassName: string): integer;
    procedure AddMsg(s: string);
  public
  end;

var
  Main: TMain;

implementation

{$R *.dfm}

uses StrUtils, rtool, rdlg, rfiletool, rDialogsEx;

procedure TMain.FormCreate(Sender: TObject);
begin
  OpenDlg.InitialDir:=GetAppDir;
  SaveDlg.InitialDir:=GetAppDir;
end;

procedure TMain.btSelDirClick(Sender: TObject);
var
  s: string;
begin
  s:=rDialogsEx.BrowseForFolderEx('', self.eDir.Text);
  if s <> '' then
    self.eDir.Text:=s;
end;

procedure TMain.AddMsg(s: string);
begin
  if s='' then
    Exit;
  if UpdMsg<>'' then
    UpdMsg:=UpdMsg+#13#10;
  UpdMsg:=UpdMsg+s;
end;

procedure TMain.btRunClick(Sender: TObject);
var
  s: string;
  I: integer;
begin
  m.Lines.clear;
  UpdFiles:=0;
  UpdProps:=0;
  // prepare dirs
  s:=Trim(self.eDir.Text);
  s:=rtool.SetEndOfPath(s);
  BkDir:=SetEndOfPath(s + '_backup_log\' + FormatDateTime('YYYYMMDD_hhmmss', Now));
  ForceDirectories(BkDir);
  // load DFM file list
  rfiletool.LoadFileListSubDir(mFList.Strings, s, '*.dfm');
  // proc all files
  for I:=0 to mFList.Strings.Count - 1 do
  begin
    ProcDFM(mFList.Strings[I], Sender=btTest);
  end;
  m.Lines.Add(Format('Files to update: %d; Properties to update: %d',[UpdFiles, UpdProps]));
  m.Lines.Add('FINISHED.');
  m.Lines.SaveToFile(BkDir + '_log.txt');
end;

procedure TMain.ProcDFM(fn: string; test: boolean);
var
  r, upd: integer;
begin
  UpdMsg:=fn; // + #13#10;
  // load DFM file
  mFile.Strings.LoadFromFile(fn);
  // proc all rows of setting grid
  upd:=0;
  for r:=1 to gr.RowCount - 2 do
  begin
    Inc(upd, ProcObj(gr.Cells[0, r], gr.Cells[1, r], gr.Cells[2, r], gr.Cells[3, r], gr.BoolCells[4, r], gr.Cells[5, r], gr.Cells[6, r]));
  end;
  if chLogFiles.Checked or (upd > 0) then
    m.Lines.Add(UpdMsg);

  if upd>0 then
  begin
    Inc(UpdProps, upd);
    Inc(UpdFiles);
  end;
  // save changes if updated
  if not test and (upd > 0) then
  begin
    // create backup
    if chBackup.Checked then
      CopyFile(PChar(fn), PChar(BkDir + ChangeFileExt(ExtractFileName(fn), '.df_bak')), true);
    // remove empty lines
    for r:=mFile.Strings.Count - 1 downto 0 do
      if mFile.Strings[r] = '' then
        mFile.Strings.Delete(r);
    // save updated DFM file
    mFile.Strings.SaveToFile(fn);
  end;
end;

function TMain.ProcObj(ClassName, PropertyName, old, new: string; del: boolean; CompName, NewClassName: string): integer;
var
  s, obj, classn, compn, prop, val: string;
  isObj, proc: boolean;
  r: integer;
begin
  Result:=0;
  isObj:=false;
  // proc all DFM lines
  for r:=0 to mFile.Strings.Count - 1 do
  begin
    s:=Trim(mFile.Strings[r]);

    if not isObj and (StrUtils.StartsText('object', s) or StrUtils.StartsText('inherited', s)) then
    begin
      // object CompName: ClassName [x]
      obj:=Trim(rtool.RemoveStrPart(s, ' ', true));
      compn:=Trim(rtool.RemoveStrPart(s, ':', true));
      s:=Trim(s);
      classn:=Trim(rtool.RemoveStrPart(s,' '));
      // s contains rest, empty or [x]
      if MatchesMask(classn, ClassName) then
      begin
        // correct classname = start of object def if comp name
        if (CompName='') or MatchesMask(compn, CompName) then
        begin
          isObj:=true;
          if NewClassName<>'' then
          begin
            mFile.Strings[r]:=Format('%s %s: %s %s', [obj, compn, NewClassName, s]);
            AddMsg(Format('Line %d, Object: %s, Class: %s  >> SET NEW CLASS: %s', [r + 1, compn, classn, NewClassName]));
            Inc(Result);
          end;
        end;
        // m.Lines.Add(Format('Object %s found on line %d', [ClassName, r + 1]))
      end;
    end;
    if isObj and SameText('end', s) then
    begin
      // end of object
      isObj:=false;
      // m.Lines.Add(Format('Object %s end on line %d', [ClassName, r + 1]))
    end;
    if isObj and (PropertyName<>'') then
    begin
      // PropName = Value
      prop:=Trim(rtool.RemoveStrPart(s, '=', true));
      val:=Trim(s);
      if MatchesMask(prop, PropertyName) then
      begin
        // compare current value
        proc:=(old = '') or MatchesMask(val, old);
        if proc then
        begin
          // correct property, log
          s:=Format('Line %d, Object: %s, Class: %s, Property %s, Value: %s  >> ', [r + 1, compn, classn, prop, val]);
          if del then
          begin
            mFile.Strings[r]:='';
            AddMsg(s + 'DELETED.');
            Inc(Result);
          end
          else
            if (new <> '') then
            begin
              mFile.Strings[r]:=Format('%s = %s', [prop, new]);
              AddMsg(s + Format('SET NEW VALUE: %s.', [new]));
              Inc(Result);
            end;
        end;
      end;
    end;
  end;
end;

procedure TMain.btClearClick(Sender: TObject);
begin
  gr.ClearEntireTable();
end;

procedure TMain.btLoadClick(Sender: TObject);
begin
  if OpenDlg.Execute then
  begin
    PSFile.LoadFile(OpenDlg.FileName);
    PSMain.ReadProperties;
  end;
end;

procedure TMain.btSaveClick(Sender: TObject);
begin
  if SaveDlg.Execute then
  begin
    PSMain.SaveProperties;
    PSFile.SaveFile(SaveDlg.FileName);
  end;
end;

end.
